/*
wgt_vis_shapely_01

james seo
www.lossless.net

draw parts of michael jackson's faces
using the glove's location and size for each frame,
then keep updating all face parts continuously.

this program is a visualization of data
from the white glove tracking project.
it requires input data files that are available from
www.whiteglovetracking.com
which must be placed in the proper data directory
specified by the dataPath variable.

in addition, it requires image files
exported from the video of michael jackson's faces.
you can download the video from google video at
http://video.google.com/videoplay?docid=3844465742609635007
and export the video file as an image sequence
using the framerate of 29.97.

this program will output 10059 tiff image files
if mustSaveOutputImages is set to true.
afterwards, you can save the image files as a quicktime movie
using the framerate of 29.97.

this program does not output the audio track.
you will have to add it yourself
after you have compiled the output movie.

based on wgt_viz_04_03

input data setup code and frame class
based on yellow_box_PROCESSING
by evan roth
www.ni9e.com
*/

import processing.video.*;

boolean canBeginDrawing = false;
boolean mustSaveOutputImages = false;

String appName = "wgt_vis_shapely";
String appVer = "01";
int appW = 640;
int appH = appW*3/4;
int vidW = appW;
int vidH = appH;
String dataPath = "/Users/james/Documents/code/data/";
String imgsPath = dataPath+"billie-jean-640x480/";
PImage[] imgs;
int imgsTotalNum;
PImage curImg;
Runtime r;

frame FM;
frame FMS[];
int comma;
String temp_f, temp_x, temp_y, temp_w, temp_h;
int f, x, y, w, h;
float t, inc;
Movie inMovie;
PFont courier;
String[] lines;

int curFnum;
int initFnum;
int imgsRequestedNum = 0;

int curSrcX, curSrcY, curSrcW, curSrcH;
int blankW = 2;

int[] maskArray;

String faceImgsPath = dataPath+"michael-jackson-faces-320x240/";
int faceImgsTotalNum = 915;
PImage[] faceImgs;
PImage facePartImg;
int[] faceImgOffsets;
int faceImgId = 0;
int curFaceFnum = 0;
int faceFnumInc = 1;

/***************************************************************
* SETUP
***************************************************************/

void setup(){
  size(appW, appH);
  stroke(0);
  fill(255);
  smooth();
  
  courier = loadFont(dataPath+"CourierNewPSMT-14.vlw");
  textFont(courier, 16);
  
  lines = loadStrings(dataPath+"wgt_data_v1.txt");
  println("number of lines in data file: " + lines.length);
  
  curFnum = 0;
  
  println("setting up input data");
  setupInputData();
  println("finished setting up input data");
  println("setting up input images");
  setupInputImages();
  println("finished setting up input images");
  
  faceImgOffsets = new int[imgsTotalNum];
  for (int i = 0; i < faceImgOffsets.length; i++) {
    faceImgOffsets[i] = 0;
  }
}

void setupInputData() {
  t = 0.0f;
  inc = 1.0f / 30.0f;

  FMS = new frame[lines.length];

  for (int i=0; i < lines.length; i++) {
    comma = 0;
    temp_f = "";
    temp_x = "";
    temp_y = "";
    temp_w = "";
    temp_h = "";

    for (int j=0; j < lines[i].length(); j++) {
      char c = lines[i].charAt(j);    //this is the char we are looking at
      if(c == ','){                   //which comma are we on
        comma++;
      }
      if(comma == 0){                //break into multiple strings
        temp_f += c;
      }else if(comma == 1){
        if(c != ','){
          temp_x += c;
        }
      }else if(comma == 2){
        if(c != ','){
          temp_y += c;
        }
      }else if(comma == 3){
        if(c != ','){
          temp_h += c;
        }
      }else if(comma == 4){
        if(c != ','){
          temp_w += c;
        }
      }
    }

    f = int(temp_f);
    x = int(temp_x);
    y = int(temp_y);
    w = int(temp_w);
    h = int(temp_h);
    t += inc;

    if (x >= 640) {
      x = 639;
    }
    if (y >= 480) {
      y = 479;
    }
    if (x+w/2 >= 640) {
      w = 639-(x-w/2);
      x = 639-w/2;
    }
    if (y+h/2 >= 480) {
      h = 479-(y-h/2);
      y = 479-h/2;
    }
    
    if (x == 0 && w == 0) {
      w = blankW;
    }
    
    FM = new frame(f, x, y, w, h, t);
    FMS[i] = FM;
    //FMS[i].print();
  }
}

void setupInputImages() {
  imgsTotalNum = lines.length;
  
  imgs = new PImage[imgsTotalNum];
  
  faceImgs = new PImage[faceImgsTotalNum];
  for (int i = 0; i < faceImgsTotalNum; i++) {
    faceImgs[i] = loadImage(faceImgsPath+"michael-jackson-faces-"+nf(i+1,3)+".jpg");
  }
}

/***************************************************************
* DRAWING
***************************************************************/

void draw() {
  background(0);
  if (!canBeginDrawing) {
    initFnum = 0;
    imgsRequestedNum = imgsTotalNum-initFnum;
    curFnum = initFnum;
    canBeginDrawing = true;
  } else {
    println("current frame: " + curFnum);
    
    for (int i = initFnum; i < curFnum; i++) {
      if (FMS[i].x > 0) {
        curSrcX = FMS[i].x;
        curSrcY = FMS[i].y;
        curSrcW = FMS[i].w;
        curSrcH = FMS[i].h;
        faceImgOffsets[i] = (int) (((float) curSrcY / appH) * (faceImgsTotalNum-1));
        faceImgId = curFaceFnum + faceImgOffsets[i];
        if (faceImgId >= faceImgsTotalNum) {
          faceImgId = (faceImgsTotalNum-1) - (faceImgId - (faceImgsTotalNum-1));
        }
        facePartImg = faceImgs[faceImgId].get((curSrcX-curSrcW/2)/2,(curSrcY-curSrcH/2)/2,curSrcW/2,curSrcH/2);
        if (maskArray == null) {
          createMaskArray(curSrcW/2,curSrcH/2);
        } else {
          resizeMaskArray(curSrcW/2,curSrcH/2);
        }
        applyMask(facePartImg,100);
        image(facePartImg,curSrcX-curSrcW/2,curSrcY-curSrcH/2,curSrcW,curSrcH);
        facePartImg = null;
      }
    }
    
    r = Runtime.getRuntime();
    r.gc();
    
    curFaceFnum += faceFnumInc;
    if (curFaceFnum >= faceImgsTotalNum) {
      faceFnumInc = -1;
      curFaceFnum += faceFnumInc;
    } else if (curFaceFnum < 0) {
      faceFnumInc = 1;
      curFaceFnum += faceFnumInc;
    }
    
    curFnum++;
    if (curFnum >= initFnum + imgsRequestedNum) {
      println("done!");
      exit();
    }
    
    if (mustSaveOutputImages) {
      save(appName + "_" + curFnum +".tif");
    }
  }
}

int getFaceImgId() {
 return (int) ((float) FMS[curFnum].y / vidH * faceImgsTotalNum);
}

/***************************************************************
* IMAGE MASK
***************************************************************/

void createMaskArray(int w, int h) {
  maskArray = new int[w*h];
}

void deleteMaskArray() {
  maskArray = null;
}

void resizeMaskArray(int w, int h) {
  if (maskArray.length < w*h) {
    maskArray = expand(maskArray,w*h);
  } else if (maskArray.length > w*h) {
    maskArray = subset(maskArray,0,w*h);
  }
}

void fillMaskArray(int val) {
  for (int i = 0; i < maskArray.length; i++) {
    maskArray[i] = val;
  }
}

void applyMask(PImage img, int v) {
  fillMaskArray(v);
  img.mask(maskArray);
  img.updatePixels();
}